!====================================================!
!======= module with subroutines and functions =======!
!====================================================!
module mysubs

use globals
implicit none	

contains

	function u(cin)
	! crra utility function for consumption
		real(dp),intent(in)	:: cin(:)
		real(dp) :: u(size(cin))
		
		if ( abs(eta-1.0_dp)<1.0e-3 )	then
			u = log(cin+cmin)
		else
			u = ( (cin+cmin)**(one-eta) ) / (one-eta)
		endif
	
	end function u

	subroutine du(cin,dout)
	! derivative of utility function
		real(dp),intent(in) :: cin(:)
		real(dp),intent(out) :: dout(:)
		
		dout = (cin+cmin)**(-eta)
	
	end subroutine du
	

    subroutine lprob (xi,x,y,p)
    !this takes in xi, and finds two points in either side of array x
    !and returns their indices (stored in y) and associated probabilities 
    ! (stored in p)
        real(8), intent(in)	    :: x(:),xi
        real(8), intent(out)	:: p(2)
        integer, intent(out)    :: y(2)
        integer					:: locl,n

        n = size(x)
        locl = maxloc(x,1,mask=xi>x)

        if (xi<=x(1)) then
	        y(1) = 1
	        y(2) = 2
	        p(1) = 1.0
	        p(2) = 0.0
        elseif (locl>=n) then 
	        locl = n-1
            y(1) = n-1
            y(2) = n
            p(1) = 0.0
            p(2) = 1.0
        else
            y(1) = locl
            y(2) = locl+1
            p(2) = (xi - x(locl))/real(x(locl+1)-x(locl))
            p(1) = 1.0 - p(2)
        end if

    end subroutine lprob
	
    subroutine lininterp (n,x,y,ni,xi,yi)
    !this does linear interpolation of (x,y) at points xi
    !requires x to be sorted in ascending order
    !extrapolates out of range

        integer, intent(in)		    :: n,ni
        real(dp), intent(in)	    :: x(n),y(n),xi(ni)
        real(dp), intent(out)	    :: yi(ni)
        real(dp), dimension(ni)	    :: xl,xh,yl,yh
        integer					    :: i,j,locl(ni)

        do i = 1,ni

	        locl(i) = maxloc(x,1,mask=xi(i)>x)
        	
	        if (xi(i)<=x(1)) then
		        locl(i) = 1
	        endif

	        if (locl(i)>=n) then 
		        locl(i) = n-1
	        end if


	        xl(i) = x(locl(i))
	        xh(i) = x(locl(i)+1)
	        yl(i) = y(locl(i))
	        yh(i) = y(locl(i)+1)
        	
	        yi(i) = yl(i) + (xi(i)-xl(i))*((yh(i)-yl(i))/(xh(i)-xl(i)))

        end do
    
    end subroutine lininterp	

    subroutine lininterp1 (n,x,y,xi,yi)
    !linear interpolation of (x,y) only at xi
    !requires x to be sorted in ascending order
    !extrapolates out of range

        integer, intent(in)		:: n
        real(dp), intent(in)	:: x(:),y(:),xi
        real(dp), intent(out)	:: yi
        real(dp)	            :: xl,xh,yl,yh
        integer					:: locl

        locl = maxloc(x,1,mask=xi>x)

        if (xi<=x(1)) then
	        locl = 1
        end if

        if (locl>=n) then 
	        locl = n-1
        end if

        xl  = x(locl)
        xh  = x(locl +1)
        yl  = y(locl)
        yh  = y(locl +1)

        yi  = yl  + (xi -xl )*((yh -yl )/(xh -xl ))

    end subroutine lininterp1


    recursive subroutine quick_sort(list, order)
    ! quick sort routine from:
    ! brainerd, w.s., goldberg, c.h. & adams, j.c. (1990) "programmer's guide to
    ! fortran 90", mcgraw-hill  isbn 0-07-000248-7, pages 149-150.
    ! modified by alan miller to include an associated integer array which gives
    ! the positions of the elements in the original order.
        implicit none
        real(dp), dimension (:), intent(in out)  :: list
        integer, dimension (:), intent(out)      :: order

        ! local variable
        integer :: i

        do i = 1, size(list)
          order(i) = i
        end do

        call quick_sort_1(1, size(list))

        contains

        recursive subroutine quick_sort_1(left_end, right_end)

        integer, intent(in) :: left_end, right_end

        !     local variables
        integer             :: i, j, itemp
        real(dp)                :: reference, temp
        integer, parameter  :: max_simple_sort_size = 6

        if (right_end < left_end + max_simple_sort_size) then
          ! use interchange sort for small lists
          call interchange_sort(left_end, right_end)

        else
          ! use partition ("quick") sort
          reference = list((left_end + right_end)/2)
          i = left_end - 1; j = right_end + 1

          do
            ! scan list from left end until element >= reference is found
            do
              i = i + 1
              if (list(i) >= reference) exit
            end do
            ! scan list from right end until element <= reference is found
            do
              j = j - 1
              if (list(j) <= reference) exit
            end do


            if (i < j) then
              ! swap two out-of-order elements
              temp = list(i); list(i) = list(j); list(j) = temp
              itemp = order(i); order(i) = order(j); order(j) = itemp
            else if (i == j) then
              i = i + 1
              exit
            else
              exit
            end if
          end do

          if (left_end < j) call quick_sort_1(left_end, j)
          if (i < right_end) call quick_sort_1(i, right_end)
        end if

        end subroutine quick_sort_1


        subroutine interchange_sort(left_end, right_end)

        integer, intent(in) :: left_end, right_end

        !     local variables
        integer             :: i, j, itemp
        real                :: temp

        do i = left_end, right_end - 1
          do j = i+1, right_end
            if (list(i) > list(j)) then
              temp = list(i); list(i) = list(j); list(j) = temp
              itemp = order(i); order(i) = order(j); order(j) = itemp
            end if
          end do
        end do

        end subroutine interchange_sort

    end subroutine quick_sort
	
    subroutine setseed(my_seed)
        implicit none 
        integer, optional ,intent(in)   :: my_seed  
        integer,allocatable             :: seed(:) 
        integer                         :: the_size,j,lengthr
              
        call random_seed(size=the_size) ! how big is the intrisic seed? 
        allocate(seed(the_size))        ! allocate space for seed 
        do j=1,the_size                 ! create the seed 
            seed(j)=abs(my_seed)+(j-1)  
        enddo 
        call random_seed(put=seed)      ! assign the seed 
        deallocate(seed)                ! deallocate space 

    end subroutine setseed
  	
end module mysubs