!===========================================!
!======= module to simulate the model =======!
!===========================================!
module mod_simulate

use globals
use mysubs
use mod_policy
implicit none	

contains

    subroutine simulate_earnings
    ! simulates earnings process for 'nsim' lifecycles
    ! in this version, we use exactly the income process in
    ! storesletten, telmer and yaron (jme 2004, appendix)
    !   -> y_t = kappa_t + alpha + z_t + zt_t
    ! where:
    !   - alpha: fixed effect/idiosyncratic effect
    !   - z: permanent effect (ar 1)
    !   - zt: transitory shock
    ! note that it is assumed that
    !     y_1 = kappa_1 + alpha + zt_1 (no perm. shock at age 1)
    
        implicit none
        integer     :: isim,iz,izt,izf,it,adsl,ia
        integer     :: ishock      ! location of shocks
        real(dp)    :: avse(nsim)
        integer     :: order(nsim)
        
        integer     :: indz(2),stata=123
        real(dp)    :: prz(2)
        real(dp)    :: meanlw, lwvar(t0:tw)

        real        :: eff_meanw(t0:tw,nzf),obj_meanw(t0:tw,nzf)  
        character(len=3)   :: izf_s      
      
        call setseed(12345)
        call random_number(esim)
        call random_number(etsim)
        call random_number(efsim)
               
        do isim = 1,nsim
            
            ! first wage shock: fixed effect component
            do izf = 1,nzf
                if ( efsim(isim)<=sum(zfprob(1:izf)) )    exit
            enddo
            izfloc(isim) = izf
            zfsim(isim)  = zfgrid(izfloc(isim))

            ! first wage shock: transitory component 
            do izt = 1,nzt
                if ( etsim(t0,isim)<=sum(ztprob(1:izt)) )    exit
            enddo
            iztloc(t0,isim) = izt
            ztsim(t0,isim)  = ztgrid(iztloc(t0,isim))
            
            ! first wage shock: permanent component 
            do iz = 1,nz
                if ( esim(t0,isim)<=sum(z0prob(1:iz)) )    exit
            enddo
            ishock = iz
            
            !----------------------------------------------!
            ! in stor2004, first perm shock is set to zero !
            !----------------------------------------------!
            izloc(t0,isim) = nint(nz/two)   !this assumes that nz is odd
            zsim(t0,isim) = zgrid(izloc(t0,isim))
                                    
            wsimout(t0,isim) = exp( lcp(t0) + zfsim(isim) + ztsim(t0,isim) + zsim(t0,isim) ) 
            
            wsim(t0,isim)    = wsimout(t0,isim)/phi_inc(t0)
            
            ! lifecycle wage shocks
            do it = t0+1,tw
              
                ! update transitory shock
                do izt = 1,nzt
                    if ( etsim(it,isim)<=sum(ztprob(1:izt)) )    exit
                enddo
                iztloc(it,isim) = izt
                ztsim(it,isim)  = ztgrid(iztloc(it,isim))
                
                ! update permanent shock
                do iz = 1,nz
                    if ( esim(it,isim)<=sum(ztrans(izloc(it-1,isim),1:iz)) ) exit
                enddo
                izloc(it,isim)  = iz
                zsim(it,isim)   = zgrid(izloc(it,isim))

                wsimout(it,isim)= exp( lcp(it) + zfsim(isim) + ztsim(it,isim) + zsim(it,isim) ) 
                                  
                wsim(it,isim)   = wsimout(it,isim)/phi_inc(it)

            enddo   ! loop it
        enddo       ! loop isim
        
        !-----------------!
        ! calculate means !
        !-----------------!
        do it=t0,tw
        do isim=1,nsim
            if (it==t0) avesim(t0,isim) = wsimout(t0,isim)
            if (it> t0) avesim(it,isim) = ((it-t0)*avesim(it-1,isim) + &
                                              wsimout(it,isim))/(it-t0+one)
        enddo
        enddo

        ! mean by age --> needed for the calculation of the pensions
        mwsim= alivew*sum(wsimout,2)/real(nsim)
        mwsim_eq=alivew*sum(wsim,2)/real(nsim)
           
        ! cross sectional mean
        cswsim =sum(mwsim)  !/(tw-t0+1)
        cswsim_eq =sum(mwsim_eq)    !/(tw-t0+1)
          
        ! store some earnings' stats and put them in globals
        do it = t0,tw
            avse = avesim(it,:)
            
            call quick_sort(avse,order)
            medwsim(it) = avse(nsim*0.5)          
            minwsim(it) = avse(nsim*0.1_dp)
            maxwsim(it) = avse(nsim*0.9_dp)

        enddo
                           
        15 format(i,10f12.1)
       
    end subroutine simulate_earnings
    
   
    subroutine simulate_lifecycle
    ! given policy functions (computed in main.f90)
    ! this subroutine simulate lifecycles of 
    ! consumption and asset accumulation
    !   note: shocks and earnings simulated externally
    !   (zsim and wsim respectively)
    !   using subroutine <simulate_earnings>

    integer     :: it,isim,indpen,iwsim
    real(dp)    :: tempg(na)
    integer     :: indsim(2)
    real(dp)    :: prsim(2)

    ! initial assets
    asim(t0,:) = 0.0_dp

    do isim = 1,nsim

    !=============================!
    !======= working stage =======!
    !=============================!

        do it = t0,tw       
            ! put avesim in avgearn grid
            ! (avesim calculated in simulate_earnings)
            call lprob(avesim(it,isim),avgearn(it,:),indsim,prsim)
            if (prsim(1)>=prsim(2))  aveloc(it,isim) = indsim(1)
            if (prsim(1)<prsim(2)) aveloc(it,isim) = indsim(2)
        
            iwsim = pind( aveloc(it,isim), &
                          izloc(it,isim),  &
                          iztloc(it,isim), &
                          izfloc(isim) )

            ! savings and consumption
            call lininterp1(na,agrid(it,izfloc(isim),:), &
                               cstrw(it,iwsim,:),        &
                               asim(it,isim),            &
                               csim(it,isim) )            

            asim(it+1,isim) = asim(it,isim)*(r/surv(it-1))+wsim(it,isim)- &
                              policy_tax(wsim(it,isim))-csim(it,isim)

            ! check borrowing constraint
            if (it<tw)  then
                tempg = agrid(it+1,izfloc(isim),:)
            elseif (it==tw) then
                call lprob(pensim(isim),pgrid,indsim,prsim)
                if (prsim(1)>=prsim(2))  indpen = indsim(1)
                if (prsim(1)<prsim(2)) indpen = indsim(2)
                tempg = agridr(it+1,indpen,:)
            endif
            
            if (asim(it+1,isim)<tempg(1))   then
                asim(it+1,isim) = tempg(1)
                csim(it,isim) = asim(it,isim)*(r/surv(it-1))+ &
                              wsim(it,isim)- policy_tax(wsim(it,isim)) - &
                              asim(it+1,isim)
            endif
            
        enddo   ! age (it) loop

        !================================!
        !======= retirement stage =======!
        !================================!

        do it = tr,tt-1
        
            call lprob(pensim(isim),pgrid,indsim,prsim)

            if (prsim(1)>=prsim(2))  indpen = indsim(1)
            if (prsim(1)<prsim(2)) indpen = indsim(2)
            
            ! savings and consumption
            call lininterp1(na,agridr(it,indpen,:), &
                               cstrr(it,indpen,:),  &
                               asim(it,isim),       &
                               csim(it,isim) )

            asim(it+1,isim) = asim(it,isim)*(r/surv(it-1))+ &
                              pgrid(indpen)/phi_inc(it) - &
                              csim(it,isim)
                         

            ! check borrowing constraint
            if (asim(it+1,isim)<agridr(it+1,indpen,1))   then
                asim(it+1,isim) = agridr(it+1,indpen,1)
                csim(it,isim) = asim(it,isim)*(r/surv(it-1))+ &
                              pgrid(indpen)/phi_inc(it) - &
                              asim(it+1,isim)
            endif

       
        enddo   ! age (it) loop 
        call lprob(pensim(isim),pgrid,indsim,prsim)
        if (prsim(1)>=prsim(2))  indpen = indsim(1)
        if (prsim(1)<prsim(2)) indpen = indsim(2)               
        csim(tt,isim) = asim(tt,isim)*(r/surv(tt-1)) + pgrid(indpen)/phi_inc(tt)

    enddo   ! sim no. (isim) loop
    
    ! estimate total consumption in the economy
    totcons = 0.0d0
    
    do it=t0,tt
        totcons = totcons + alive(it)*sum(csim(it,:))
    enddo

    end subroutine simulate_lifecycle
    
end module mod_simulate
