!======================================================!
!======= this subroutine solves the model using =======!
!======= the endogenous grid method ===================!
!======================================================!

module mod_solveit

use globals
use mod_policy
use mysubs

contains 

subroutine solveit

    implicit none
    
	integer     :: it,ia,ic,iap,icp,iz,iz1,ie,i10,i11
	integer     :: iztp,izp,iwp(2)
	integer     :: ij,izt,izf,ir,iw
	real(dp)    :: temp(na),tempv(na),tempg(na),ctemp(na)
	real(dp)    :: wage,wageout,wpr
	integer     :: inext(2)                     ! to put resulting average earnings/pensions
	real(dp)    :: vpnext,pnext(2)              ! back into avg earnings/pensions grid

    cstrr = 0.0_dp
    cstrw = 0.0_dp

!    print*, 'solving the life-cycle...'

    !======= last period =======!
    do ir = 1,nr
        ! consumption
        ! max operator because of numerical imprecision
        cstrr(tt,ir,:) = max((r/surv(tt-1))*agridr(tt,ir,:) + pgrid(ir)/phi_inc(tt),0d0)
        
        ! marginal utility of consumption
        call du(cstrr(tt,ir,:),ducstrr(tt,ir,:))
        
    enddo
        
    !================================!
    !======= retirement stage =======!
    !================================!
    do it = tt-1,tr,-1
    do ir = 1,nr

        tempv = 0.0_dp      

        tempv = tempv+fam1(it)*ducstrr(it+1,ir,:)

        tempv = beta*r*tempv
        tempg = agridr(it+1,ir,:)
        
        ctemp = fam2(it)*tempv**(-one/eta)
        yendr = ctemp + tempg - pgrid(ir)/phi_inc(it)
        agridr0 = yendr/(r/surv(it-1))

        ! check borrowing limits
        j = 1
        do while ( agridr(it,ir,j) .lt. agridr0(1))
            ! max operator because of numerical imprecision
            cstrr(it,ir,j) = max(agridr(it,ir,j)*(r/surv(it-1)) + pgrid(ir)/phi_inc(it) - &
                                 agridr(it+1,ir,1),0d0)
            j = j+1
            if (j>na) exit
        enddo

        ! calculate endogenous grid
        call lininterp(na,agridr0,ctemp, &
                       na-j+1,agridr(it,ir,j:na),cstrr(it,ir,j:na) )

        call du(cstrr(it,ir,:),ducstrr(it,ir,:))

    enddo   ! ir (pension) loop        
    enddo   ! it (age) loop
          
    !=============================!
    !======= working stage =======!
    !=============================!
    do it = tw,t0,-1
    do ir = 1,nr
    do iz = 1,nz
    do izt = 1,nzt
    do izf = 1,nzf

        iw = pind(ir,iz,izt,izf)
        
        wageout = exp( lcp(it) + zfgrid(izf) &
                               + ztgrid(izt) &
                               + zgrid(iz) )
       
        wage = wageout/phi_inc(it)
        
        ! given avg. earnings so far and new wage
        ! calculate expected mg. util of cons inside bounds of 
        ! the average earnings/pension grid

        tempv = 0.0_dp

        if (it == tw)   then
            
            vpnext = pension( avgearn(tw,ir) )
            call lprob(vpnext,pgrid,inext,pnext)
            
            tempv = tempv+fam1(it)* &
                (pnext(1)*ducstrr(tr,inext(1),:) + &
                 pnext(2)*ducstrr(tr,inext(2),:))                    
                                     
            tempg = agridr(tr,ir,:) 

        else if (it < tw)   then
            
            do izp = 1,nz
            do iztp = 1,nzt
                wpr = exp( lcp(it+1) + zfgrid(izf) &
                         + ztgrid(iztp) &
                         + zgrid(izp) )
                         
                wpr = wpr/phi_inc(it+1)

                vpnext = ( (it+one-t0)*avgearn(it,ir) + wpr ) / (it+one-t0+one)
                call lprob(vpnext,avgearn(it+1,:),inext,pnext)
            
                iwp(1) = pind(inext(1),izp,iztp,izf)
                iwp(2) = pind(inext(2),izp,iztp,izf)
            
                tempv = tempv+fam1(it) * &
                        ztrans(iz,izp) * &
                        ztprob(iztp)   * &
                       (pnext(1)*ducstrw(it+1,iwp(1),:)+ &
                        pnext(2)*ducstrw(it+1,iwp(2),:))
                        
            enddo
            enddo

            tempg = agrid(it+1,izf,:)

        endif

        tempv=beta*r*tempv
       
        ctemp = fam2(it)*tempv**(-one/eta)
        yendw = ctemp + tempg - wage + policy_tax(wage)
        agrid0 = yendw/(r/surv(it-1))

        ! check borrowing limits
        j = 1
        do while ( agrid(it,izf,j) .lt. agrid0(1) )
            ! max operator because of numerical imprecision
            cstrw(it,iw,j) = max(agrid(it,izf,j)*(r/surv(it-1)) + wage - policy_tax(wage) - tempg(1),0d0)
            j = j+1
            if (j>na) exit
        enddo

        ! calculate endogenous grid
        call lininterp(na,agrid0,ctemp, &
                       na-j+1,agrid(it,izf,j:na),cstrw(it,iw,j:na) )
                       
        ! update 
        call du(cstrw(it,iw,:),ducstrw(it,iw,:))
        
    enddo   ! izf (fixed effect shock) loop
    enddo   ! izt (trans. wage shock) loop
    enddo   ! iz (perm. wage shock) loop
    enddo   ! ir (average earnings) loop    
    enddo   ! it (age) loop

!    print*, "done with model solutions..."


end subroutine solveit

end module mod_solveit